@@ -244,17 +244,7 @@ typedef struct transport_context {
 
   /* items below only used with option topt_use_bdat */
   tpt_chunk_cmd_cb	  chunk_cb;		/* per-datachunk callback */
-  struct smtp_inblock	* inblock;
-  struct smtp_outblock	* outblock;
-  host_item		* host;
-  struct address_item	* first_addr;
-  struct address_item	**sync_addr;
-  BOOL			  pending_MAIL;
-  BOOL			  pending_BDAT;
-  BOOL			  good_RCPT;
-  BOOL			* completed_address;
-  int			  cmd_count;
-  uschar		* buffer;
+  void			* smtp_context;
 } transport_ctx;
 
 
@@ -635,7 +625,7 @@ typedef struct address_item {
                                   /* (may need to hold a timestamp) */
 
   short int basic_errno;          /* status after failure */
-  short int child_count;          /* number of child addresses */
+  unsigned short child_count;     /* number of child addresses */
   short int return_file;          /* fileno of return data file */
   short int special_action;       /* ( used when when deferred or failed */
                                   /* (  also  */
